class Skill < ActiveRecord::Base
  has_ancestry

  # model attributes
  attr_accessible :name, :subject_ids, :parent, :parent_id

  validates_presence_of :name
  validates_uniqueness_of :name

  # model associations
  has_many :skill_progressions, :dependent => :destroy
  has_many :users, :through => :skill_progressions
  has_many :tasks, :dependent => :destroy
  has_and_belongs_to_many :subjects,
    :before_add => :validates_subject,
    :after_add => :update_children

  # custom validation
  validate :parent_subjects

  # callbacks
  before_destroy { subjects.clear }
  after_save :update_progression
  after_update :children_subjects

  accepts_nested_attributes_for :subjects
  accepts_nested_attributes_for :skill_progressions


  # validates that the created / updated skill's children have all the
  # associations to subjects that the parent skill does.
  def children_subjects
    return unless has_children?
    children.each do |child|
      child.subject_ids = subject_ids
      child.children_subjects
    end
  end

  #constraint for unique HABTM row in the join table
  def validates_subject(subject)
    raise ActiveRecord::Rollback if subjects.include? subject
  end

  # called after adding a new subject to skill.subjects.
  # If this skill has just been created then the method simply
  # returns due to ancestry not being initialised yet.
  def update_children(subject)
    return if self.new_record?
    children_subjects if has_children?
  end

  # validates that the created / updated skill still has all the
  # associations to subjects that the parent skills do.
  def parent_subjects
    return unless parent
    subject_ids = subjects.map{|x| x.id}
    parent_ids = parent.subjects.map{|x| x.id}
    if parent_ids - subject_ids != [] || subject_ids - parent_ids != []
      errors.add(:subjects, 'Denne skillen har knyttet andre fag til seg enn sin rot-skill')
    end
  end

  # called after a skill is saved in order to update
  # the associated skill_progression
  def update_progression
    SkillProgression.where(:skill_id => id).each do |progression|
      progression.update_progress
    end
    if parent
      parent.update_progression
    end
  end

  # methods called from controllers and views
  def tasks?
    tasks.length > 0
  end

  def subjects?
    subjects.length > 0
  end

  def has_subject?(subject)
    subjects.include? subject
  end

  def get_progression(user)
    if user.has_progression?(self)
      user.skill_progressions.find_by_skill_id(id).get_progress
    else
      0
    end
  end

end
