class User < ActiveRecord::Base
  has_merit

  devise :database_authenticatable, :registerable,
         :recoverable, :rememberable, :trackable, :validatable, :confirmable

  # model attributes
  attr_accessible :email, :password, :password_confirmation, :remember_me, :username, :first_name, :last_name,
                  :role_ids, :subject_ids, :teaching_class_ids, :school_class_id

  validates_uniqueness_of :username, :allow_nil => true, :case_sensitive => true
  validates_presence_of :email, :first_name, :last_name

  # model associations
  has_many :answers, :dependent => :destroy
  has_many :tasks, :through => :answers
  has_many :skill_progressions, :dependent => :destroy
  has_many :skills, :through => :skill_progressions
  accepts_nested_attributes_for :skill_progressions
  has_and_belongs_to_many :roles
  belongs_to :school_class
  has_many :teaching_classes, :foreign_key => 'teacher_id', :class_name => 'SchoolClass'


  def role?(role)
    return !!self.roles.find_by_name(role.to_s)
  end

  def has_progression?(skill)
    SkillProgression.where(:skill_id => skill.id, :user_id => id).first
  end

  def add_progression(skill)
    # adds the current skill, and any parent skills, to skill_progressions
    unless skill.is_root?
      add_progression(skill.parent)
    end
    skill_progressions << SkillProgression.new(:skill_id => skill.id, :user_id => self.id) unless has_progression?(skill)
  end

  def update_progression(skill)
    # saves status
    skill_progressions.find_by_skill_id(skill.id).update_progress
  end

  def full_name
    (first_name + last_name).titleize
  end

  def self.students
    find :all,:include => :roles, :conditions => [" roles.name = ?", "student" ]
  end

  # returns all students not assigned to a class
  def self.rogue_students
    find :all,:include => :roles, :conditions => [" roles.name = ? AND school_class_id IS NULL", "student" ]
  end

  def self.teachers
    find :all,:include => :roles, :conditions => [" roles.name = ?", "teacher" ]
  end

  def completed_task?(task)
    answers.each do |answer|
      return true if answer.task == task && answer.correct?
    end
    false
  end

  def completed_skill?(skill)
    begin
      skill_progressions.where(:skill_id => skill.id).first.complete?
    rescue
      false
    end
  end

  def attempted_task?(task)
    answers.each do |answer|
      return true if answer.task == task
    end
    false
  end

end
